package sk.fiit.jim;

import java.util.HashMap;
import java.util.Map;

import sk.fiit.jim.code_review.ReviewOk;

/**
 *  Settings.java
 *  
 *  Encapsulates global settings that alter the behaviour of the code
 *  throughout the entire project.
 *  
 *  Is meant to be set in ./scripts/config/settings.rb
 *
 *@Title        Jim
 *@author       $Author: marosurbanec $
 */
@ReviewOk
public final class Settings{
	
	private Settings(){}
	
	private static Map<String, Object> settings = new HashMap<String, Object>();
	
	static{
		setDefaults();
	}

	private static void setDefaults(){
		settings.put("kalmanUseFilter", true);
		settings.put("kalmanDefaultQ", .475);
		settings.put("kalmanDefaultR", .375);
		settings.put("runGcOnPhaseStart", true);
		settings.put("runGui", true);
		settings.put("gravityAcceleration", 9.81);
		settings.put("maximumAngularChangePerQuantum", 7.0);
	}
	
	public static void setValue(String key, Object value){
		settings.put(key, value);
	}
	
	public static double getDouble(String key){
		return (Double)get(key);
	}
	
	public static boolean getBoolean(String key){
		return (Boolean)get(key);
	}

	private static Object get(String key){
		Object setting = settings.get(key);
		if (setting == null)
			throw new IllegalStateException("Setting: "+key+" not found");
		return setting;
	}
}