package sk.fiit.jim.agent;

import org.junit.Before;
import org.junit.Test;

import static java.lang.Math.PI;

import static org.hamcrest.CoreMatchers.is;

import static org.hamcrest.Matchers.closeTo;
import static org.hamcrest.Matchers.lessThan;

import static org.junit.Assert.assertThat;

import sk.fiit.jim.agent.models.AgentModel;
import sk.fiit.jim.agent.parsing.Parser;
import sk.fiit.jim.math.Vector3D;

/**
 *  ParserToAgentModelIntergrationTest.java
 *
 *@Title        Jim
 *@author       $Author: marosurbanec $
 */
public class ParserToAgentModelIntegrationTest{
	
	String message = 
	"(See (G2R (pol 10.05 -6.14 1.41)) (G1R (pol 10.05 5.94 1.49)) (F1R (pol 11.68 31.06 -2.52)) (F2R (pol 11.66 -31.10 -2.73)) (B (pol 1.11 -0.38 -26.39)))";
	private AgentModel agent;
	
	@Before
	public void setup(){
		agent = new AgentModel();
		Parser.clearObservers();
		Parser.subscribe(agent);
	}
	
	@Test
	public void testRotationAndPositioncalculation(){
		new Parser().parse(message);
		assertGoodRotation();
		assertGoodPosition();
	}

	private void assertGoodPosition(){
		Vector3D expected = Vector3D.cartesian(-2.0, 0.0, 0.5);
		Vector3D diff = agent.getPosition().subtract(expected);
		assertThat(diff.getR(), is(lessThan(0.1)));
	}

	private void assertGoodRotation(){
		double expectedX = 0.0;
		double expectedY = 0.0;
		double expectedZ = 1.5 * PI;
		assertThat(agent.getRotationX(), is(closeTo(expectedX, 0.1)));
		assertThat(agent.getRotationY(), is(closeTo(expectedY, 0.1)));
		assertThat(agent.getRotationZ(), is(closeTo(expectedZ, 0.1)));
	}
}