package sk.fiit.jim.agent.communication;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;

import static sk.fiit.jim.log.LogType.INIT;

import sk.fiit.jim.agent.AgentInfo;
import sk.fiit.jim.agent.Planner;
import sk.fiit.jim.agent.parsing.Parser;
import sk.fiit.jim.code_review.Problem;
import sk.fiit.jim.log.Log;
import sk.fiit.jim.log.LogType;

/**
 * Communication.java
 * 
 * Takes care about the low-level communication with the server - receives and
 * delivers messages.
 * 
 * @Title Jim
 * @author $Author: marosurbanec $
 */
@Problem("Document methods")
public class Communication {
	private static final String BYE = "(bye)";
	byte[] buffer = new byte[5120];
	Socket socket;
	String serverIp;
	Integer port;
	StringBuilder outMessageBuffer = new StringBuilder();

	private static Communication instance = new Communication();
	private DataInputStream input;
	private DataOutputStream output;
	private Parser parser = new Parser();

	public static Communication getInstance() {
		return instance;
	}

	@Problem("Document!")
	public void start() throws UnknownHostException, IOException {
		try {
			Log.log(INIT, "Attempting to log to: %s:%d", serverIp, port);
			socket = new Socket(serverIp, port);
			input = new DataInputStream(socket.getInputStream());
			output = new DataOutputStream(socket.getOutputStream());
			registerSayingByeOnExit();
			handshake();
		} catch (Exception e) {
			Log.error(LogType.INIT, "Unable to connect. Cause: %s",
					e.getMessage());
			System.exit(-1);
		}

		mainLoop();
	}

	private void registerSayingByeOnExit() {
		Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
			public void run() {
				try {
					transmit(BYE);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}));
	}

	private void mainLoop() throws IOException {
		while (true) {
			while (input.available() == -1) {
				Thread.yield();
			}
			String incoming = receive();
			parser.parse(incoming);
			Planner.proceed();
			transmit(outMessageBuffer.append("(syn)").toString());
			outMessageBuffer = new StringBuilder();
		}
	}

	private void handshake() throws IOException {
		transmit("(scene rsg/agent/nao/nao.rsg)");

		String handShake = receive();
		System.out.println(handShake);
		String initMessage = String.format("(init (unum %d)(teamname %s))",
				AgentInfo.playerId, AgentInfo.team);
		transmit(initMessage);
	}

	@Problem("Document!")
	public void addToMessage(String message) {
		outMessageBuffer.append(message);
	}

	private void transmit(String what) throws IOException {
		Log.log(LogType.OUTCOMING_MESSAGE, what);
		output.writeInt(what.length());
		output.write(what.getBytes());
		output.flush();
	}

	private String receive() throws IOException {
		int messageLength = input.readInt();
		
		if (Math.abs(messageLength) > buffer.length) {
			return "";
		}

		int bytesAlreadyRead = 0;
		while (messageLength > bytesAlreadyRead)
			bytesAlreadyRead += input.read(buffer, bytesAlreadyRead,
					messageLength - bytesAlreadyRead);

		String incoming = new String(buffer, 0, messageLength);
		Log.log(LogType.INCOMING_MESSAGE, incoming);
		return incoming;
	}

	public void setServerIp(String serverIp) {
		this.serverIp = serverIp;
	}

	public void setPort(Integer port) {
		this.port = port;
	}
}