package sk.fiit.jim.agent.models;

import sk.fiit.jim.code_review.Problem;
import sk.fiit.jim.math.Vector3D;

/**
 *  DynamicObject.java
 *
 *@Title        Jim
 *@author       $Author: marosurbanec $
 */
@Problem({"Predicts position without any friction (maybe Ball overrides getPosition?)",
	"document!"})
public class DynamicObject{

	private Vector3D position;
	private Vector3D relativePosition;
	private Vector3D speed;
	private double lastTimeSeen;

	public void setRelativePosition(Vector3D relativePosition){
		this.relativePosition = relativePosition;
	}
	
	public void setPosition(Vector3D absolutePosition, double when){
		double timeElapsed = when - lastTimeSeen;
		if (this.position != null)
			speed = absolutePosition.subtract(this.position).divide(timeElapsed);
		this.lastTimeSeen = when;
		this.position = absolutePosition;
	}
	
	public Vector3D getPosition(){
		return position;
	}

	public Vector3D getRelativePosition(){
		return relativePosition;
	}
	
	public Vector3D predictPositionIn(double offset){
		if (offset < 0)
			throw new IllegalArgumentException(String.format("Negative offset supplied: %.2f", offset));
		return speed.multiply(offset).add(position);
	}
	
	public Vector3D predictRelativePositionIn(double offset){
		AgentModel agentModel = AgentModel.getInstance();
		
		Vector3D withUnsyncedRotations = predictPositionIn(offset).
			rotateOverX(-agentModel.getRotationX()).
			rotateOverY(-agentModel.getRotationY()).
			rotateOverZ(-agentModel.getRotationZ());
		return withUnsyncedRotations.subtract(agentModel.getPosition());
	}
	
	public double getLastTimeSeen(){
		return lastTimeSeen;
	}
	
	@Override
	public String toString(){
		return new StringBuilder().
			append("Absolute: ").
			append(position).
			append('\n').
			append("Relative: ").
			append(relativePosition).
			toString();
	}
}