package sk.fiit.jim.agent.models;

import sk.fiit.jim.agent.parsing.ParsedData;
import sk.fiit.jim.agent.parsing.ParsedDataObserver;
import sk.fiit.jim.code_review.UnderConstruction;
import sk.fiit.jim.math.Vector3D;

/**
 *  WorldModel.java
 *  
 *  Data holder for objects on the playground. Carries information
 *  about their respective position, velocity and their average observed velocity.
 *
 *@Title        Jim
 *@author       $Author: marosurbanec $
 */
@UnderConstruction(todo={"Position of other agents", "Documentation"})
public class WorldModel implements ParsedDataObserver{
	
	private DynamicObject ball = new DynamicObject();
	private AgentModel agentModel;
	private static WorldModel instance = new WorldModel();
	
	public WorldModel(){
		this.agentModel = AgentModel.getInstance();
	}
	
	public static WorldModel getInstance(){
		return instance;
	}
	
	public void processNewServerMessage(ParsedData data){
		if (data.ballRelativePosition != null)
			calculateBallPosition(data);
	}
	
	private void calculateBallPosition(ParsedData data){
		ball.setRelativePosition(data.ballRelativePosition);
		Vector3D withSyncedRotations = data.ballRelativePosition.
			rotateOverX(agentModel.getRotationX()).
			rotateOverY(agentModel.getRotationY()).
			rotateOverZ(agentModel.getRotationZ());
		Vector3D absolutePosition = agentModel.getPosition().add(withSyncedRotations);
		ball.setPosition(absolutePosition, data.SIMULATION_TIME);
	}

	public DynamicObject getBall(){
		return ball;
	}
	
	public void setAgentModel(AgentModel agent){
		this.agentModel = agent;
	}
}