package sk.fiit.jim.agent.models;

import org.junit.Before;
import org.junit.Test;

import sk.fiit.jim.agent.parsing.ParsedData;
import sk.fiit.jim.math.Vector3D;

import static java.lang.Math.PI;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.CoreMatchers.not;
import static org.hamcrest.CoreMatchers.nullValue;

import static org.junit.Assert.assertThat;

/**
 *  WorldModelTest.java
 *
 *@Title        Jim
 *@author       $Author: marosurbanec $
 */
public class WorldModelTest{
	
	WorldModel model;
	private AgentModel agent;
	
	@Before
	public void setup(){
		agent = new AgentModel();
		model = new WorldModel();
		model.setAgentModel(agent);
	}
	
	
	@Test
	public void ballPositions(){
		seeBall();
		assertThat(model.getBall(), is(not(nullValue())));
		assertThat(model.getBall().getPosition(), is(not(nullValue())));
		assertThat(model.getBall().getPosition(), is(equalTo(Vector3D.cartesian(-1.0, 1.0, 0.0))));
	}


	private void seeBall(){
		seeBallIn(1.0, 1.0, 0.0);
	}


	private void seeBallIn(double x, double y, double z){
		ParsedData data = new ParsedData();
		data.fixedObjects.put(FixedObject.OUR_LOWER_CORNER, FixedObject.OUR_LOWER_CORNER.getAbsolutePosition().rotateOverZ(-PI / 2.0));
		data.fixedObjects.put(FixedObject.OUR_UPPER_POST, FixedObject.OUR_UPPER_POST.getAbsolutePosition().rotateOverZ(-PI / 2.0));
		data.fixedObjects.put(FixedObject.OUR_LOWER_POST, FixedObject.OUR_LOWER_POST.getAbsolutePosition().rotateOverZ(-PI / 2.0));
		
		data.ballRelativePosition = Vector3D.cartesian(x, y, z);
		agent.processNewServerMessage(data);
		model.processNewServerMessage(data);
	}
}