package sk.fiit.jim.agent.moves;

import static sk.fiit.jim.log.LogType.LOW_SKILL;
import sk.fiit.jim.agent.communication.Communication;
import sk.fiit.jim.agent.models.EnvironmentModel;
import sk.fiit.jim.log.Log;
import sk.fiit.jim.log.LogType;

/**
 * LowSkill.java
 * 
 * @Title Jim
 * @author $Author: marosurbanec $
 */
public class LowSkill {

	public String name;
	public String initialPhase;
	Phase activePhase;
	private double nextTimePhaseEnds = 0.0;

	public LowSkill() {}

	public LowSkill(String name, String initialPhase) {
		
		if (name == null) {
			throw new IllegalArgumentException("LowSkill constructor argument 1(String name) is null");
		}
		
		if (initialPhase == null) {
			throw new IllegalArgumentException("LowSkill constructor argument 2(String initialPhase) is null");
		}
		
		this.name = name;
		this.initialPhase = initialPhase;
	}

	public void executeFinalisation() {
		
		if (!activePhase.isFinal || !phaseEnded()) {
			return;
		}

		setNewActivePhase(activePhase.finalizationPhase);
	}

	public void reset() {
		this.activePhase = null;
		this.nextTimePhaseEnds = 0.0;
	}

	public void carryOn() {
		if (activePhase == null) {
			setNewActivePhase(initialPhase);
		}
		if (phaseEnded() && activePhase.next != null) {
			setNewActivePhase(activePhase.next);
		}
		if (phaseEnded() && activePhase.next == null) {
			setNewActivePhase(Phase.EMPTY_PHASE.name);
		}
		
		//zatial komentujem bo to vypisuje vela textu - edo
		//Log.debug(LOW_SKILL, "Currently active phase: %s", activePhase.name);
		JointPlacement.generateMessage();
		Communication.getInstance().addToMessage(JointPlacement.lastMessage);
	}

	private void setNewActivePhase(String phaseName) {
		activePhase = Phases.get(phaseName);
		nextTimePhaseEnds = EnvironmentModel.SIMULATION_TIME + activePhase.duration;
		Log.log(LogType.LOW_SKILL, "Starting phase %s", activePhase.name);
		JointPlacement.calculateNewTargetState(activePhase);
	}

	public boolean ended() {
		return phaseEnded() && activePhase != null && activePhase.isFinal;
	}

	private boolean phaseEnded() {
		return (EnvironmentModel.SIMULATION_TIME - nextTimePhaseEnds) > -0.01d;
	}

	@Override
	public String toString() {
		return name;
	}
}