package sk.fiit.jim.agent.skills;

import static sk.fiit.jim.log.LogType.HIGH_SKILL;
import sk.fiit.jim.agent.moves.LowSkill;
import sk.fiit.jim.log.Log;

/**
 *  HighSkill.java
 *
 *@Title        Jim
 *@author       $Author: marosurbanec $
 */
public abstract class HighSkill{
	
	protected LowSkill currentSkill;
	protected boolean finalizing = false;
	private boolean ended = false;
	
	public final void execute() throws Exception{
		Log.debug(HIGH_SKILL, "Currently executing: %s", currentSkill);
		if (currentSkill != null && !currentSkill.ended()){
			checkProgress();
			currentSkill.carryOn();
			return;
		}
		
		LowSkill newSkill = pickLowSkill();
		if (currentSkill != null && !finalizing && (newSkill == null || newSkill != currentSkill)){
			currentSkill.executeFinalisation();
			currentSkill.carryOn();
			finalizing = true;
		}
		
		if (finalizing && currentSkill.ended())
			finalizing = false;
		
		if (currentSkill != newSkill && newSkill != null){
			newSkill.reset();
			newSkill.carryOn();
		}
		
		if (!finalizing)
			currentSkill = newSkill;
		
		ended = (currentSkill == null);//currentSkill.ended();
	}
	
	/**
	 * Return the next skill that needs to be executed
	 * in order to fulfill goals of this HighSkill.
	 * 
	 * If no skill is returned (returns null), a finalization
	 * of the last one is executed. If finalization has succesfully
	 * completed, this HighSkill's instance is abandoned and no longer
	 * used
	 * 
	 * @return {@link LowSkill} next skill that needs to be executed. Null if the goal is met
	 */
	public abstract LowSkill pickLowSkill();
	
	/**
	 * Throw an exception if robot is found to be in
	 * an unexpected state, e.g. fallen on ground during walking.
	 * Causes this skill execution to be immediately terminated.
	 * 
	 * @throws Exception any kind of exception will do (because of JRuby's hierarchy)
	 */
	public abstract void checkProgress() throws Exception;
	
	public boolean isEnded(){
		return ended;
	}

	@Override
	public String toString(){
		return "Current skill "+currentSkill;
	}
}